#! /bin/sh
# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

# Copyright (c) 1998 America Online, Inc. All Rights Reserved.
#
# AOL grants you ("Licensee") a non-exclusive, royalty free, license to use,
# modify and redistribute this software in source and binary code form,
# provided that i) this copyright notice and license appear on all copies of
# the software; and ii) Licensee does not utilize the software in a manner
# which is disparaging to AOL.
#
# This software is provided "AS IS," without a warranty of any kind. ALL
# EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
# IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
# NON-INFRINGEMENT, ARE HEREBY EXCLUDED. AOL AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL AOL OR ITS
# LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
# INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
# CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
# OR INABILITY TO USE SOFTWARE, EVEN IF AOL HAS BEEN ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGES.
#
# This software is not designed or intended for use in on-line control of
# aircraft, air traffic, aircraft navigation or aircraft communications; or in
# the design, construction, operation or maintenance of any nuclear
# facility. Licensee represents and warrants that it will not use or
# redistribute the Software for such purposes.

# configTool -
# Signon to toc, send config from a file, and sign off.
# Currently this is a simple Tcl application, someday
# it might be a nice Tk app.  Also would be nice
# if it could handle importing Java .aim/AIM.cfg files.

# Make sure we are in the tik directory and load the toc routines.
cd [file dirname $argv0]
catch {cd [file dirname [file readlink $argv0]]}
source toc.tcl

########### GLOBALS ###########

set TOCHOST  toc.oscar.aol.com
set TOCPORT  5190

set REVISION {configTool.tcl:$Revision: 1.6 $}

########### EVENTS ###########

proc put_signon {name data} {
    puts "Signed on."

    toc_init_done $name
    set file [open "$::FILENAME" "r"]
    set config [read $file]
    toc_set_config $name $config
    close $file

    puts "Sent Config from file '$::FILENAME'."

    after 1000
    toc_close $name

    puts "Quiting."
}

proc get_config {name data} {
    puts "Got config from TOC."

    set file [open "$::FILENAME" "w"]
    puts -nonewline $file $data
    close $file

    puts "Saved config to file '$::FILENAME'."

    after 1000
    toc_close $name

    puts "Quiting."
}

########### MAIN ###########

# Register which events we are interested in

if {($argc < 3) || ($argc > 4)} {
    puts "Usage: $argv0 (get|put) <username> <password> { <filename> }" 
    exit 1
}

set func [string tolower [lindex $argv 0]]
if {$func == "get"} {
    toc_register_func * CONFIG    get_config
} elseif {$func == "put"} {
    toc_register_func * SIGN_ON   put_signon
} else {
    puts "Usage: $argv0 (get|put) <username> <password> { <filename> }" 
    exit 1
}

set user [normalize [lindex $argv 1]]
set pass [lindex $argv 2]

if {$argc == 4} {
    set ::FILENAME [lindex $argv 3]
} else {
    set ::FILENAME $user
}

toc_open $user $TOCHOST $TOCPORT login.oscar.aol.com 5190 \
         $user $pass english $::REVISION

# Block forever and handle all the events
catch {vwait DONE}
