# Away Package
#
# All packages must be inside a namespace with the
# same name as the file name.
#
# $Revision: 1.14 $

# Options the user might want to set.  A user should use
# set ::TIK(options,...), not the tik_default_set

# How many times do we send an away message?
tik_default_set options,Away,sendmax 1

namespace eval away {

    variable info

    # Must export at least: load, unload, goOnline, goOffline
    namespace export load unload goOnline goOffline register

    # All packages must have a load routine.  This should do most
    # of the setup for the package.  Called only once.
    proc load {} {
        toc_register_func * IM_IN away::IM_IN

        set away::info(msg)      ""
        set away::info(sendaway) 0

        menu .awayMenu -tearoff 0
        .toolsMenu add cascade -label "Away Message" -menu .awayMenu
        .awayMenu add command -label "New Away Message" \
                              -command away::create_newaway
        .awayMenu add separator
    }

    # All pacakges must have goOnline routine.  Called when the user signs
    # on, or if the user is already online when packages loaded.
    proc goOnline {} {
    }

    # All pacakges must have goOffline routine.  Called when the user signs
    # off.  NOT called when the package is unloaded.
    proc goOffline {} {
    }

    # All packages must have a unload routine.  This should remove everything 
    # the package set up.  This is called before load is called when reloading.
    proc unload {} {
        toc_unregister_func * IM_IN away::IM_IN
        .toolsMenu delete "Away Message"
        destroy .awayMenu
    }


    proc IM_IN {name source msg auto} {
        if {$away::info(sendaway)} {
            set nsrc [normalize $source]
            # Don't send away message more than max times to the same person
            if {![info exists away::info(sentto,$nsrc)]} {
                set away::info(sentto,$nsrc) 0
            }

            if {$away::info(sentto,$nsrc) < $::TIK(options,Away,sendmax)
                || $::TIK(options,Away,sendmax) == -1} {
                toc_send_im $::NSCREENNAME $source \
                    [away::expand $away::info(msg) $source] auto
                incr away::info(sentto,$nsrc)
            } 
        }
    }

    proc expand {msg nick} {
        regsub -all -- "%n" $msg $nick new_msg
        regsub -all -- "%N" $new_msg $::SCREENNAME new_msg
        return $new_msg
    }

    proc back {} {
        away::set_away
        catch {destroy .awaymsg}
    }

    proc set_away {{awaymsg "_NOAWAY_"}} {
        if {$awaymsg == "_NOAWAY_"} {
            foreach i [array names away::info "sentto,*"] {
                unset away::info($i)
            }
            set away::info(sendaway) 0
            return
        } 

        set away::info(msg) $awaymsg
        set away::info(sendaway) 1

        set w .awaymsg

        if {[winfo exists $w]} {
            raise $w
            $w.text configure -state normal
            $w.text delete 0.0 end
            $w.text insert end $awaymsg
            $w.text configure -state disabled
            return
        }

        toplevel $w -class Tik
        wm title $w "Away Message"
        wm iconname $w "Away Message"
        if {$::TIK(options,windowgroup)} {wm group $w .login}

        text  $w.text -width 40 -height 8 -wrap word
        $w.text insert end $awaymsg
        $w.text configure -state disabled

        button $w.back -text "I'm Back" -command away::back

        pack $w.back -side bottom
        pack $w.text -fill both -expand 1 -side top
    }

    proc register {awaymsg} {
        catch {.awayMenu delete $awaymsg}
        .awayMenu add command -label $awaymsg -command [list away::set_away $awaymsg]
    }

    proc newaway_ok {} {
        if {![winfo exists .newaway.text]} {
            return
        }
        set awaymsg [string trim [.newaway.text get 0.0 end]]
        away::register $awaymsg
        away::set_away $awaymsg
        destroy .newaway
    }

    proc create_newaway {} {
        set w .newaway

        if {[winfo exists $w]} {
            raise $w
            return
        }

        toplevel $w -class Tik
        wm title $w "New Away Message"
        wm iconname $w "New Away Message"
        if {$::TIK(options,windowgroup)} {wm group $w .login}

        text  $w.text -width 40 -height 8 -wrap word

        label $w.info -text "This will NOT change your $::TIK(rcfile)\n Use: away::register <msg>"
        frame $w.buttons
        button $w.ok -text "Ok" -command "away::newaway_ok"
        button $w.cancel -text "Cancel" -command [list destroy $w]
        pack $w.ok $w.cancel -in $w.buttons -side left -padx 2m

        pack $w.info -side top
        pack $w.buttons -side bottom
        pack $w.text -fill both -expand 1 -side top
    }
}

# Hack
proc tik_register_away {msg} {
    puts "You need to changed your tikrc to use away::register instead of tik_register_away"
    return [away::register $msg]
}
