# Socks Proxy
#
# All packages must be inside a namespace with the
# same name as the file name.
#
# $Revision: 1.4 $

namespace eval sslproxy {

    # Must export at least: load, unload, goOnline, goOffline
    namespace export load unload goOnline goOffline

    # All packages must have a load routine.  This should do most
    # of the setup for the package.  Called only once.
    proc load {} {
        tik_register_proxy "SSL/HTTP" sslproxy::connect sslproxy::config
    }

    # All pacakges must have goOnline routine.  Called when the user signs
    # on, or if the user is already online when packages loaded.
    proc goOnline {} {
    }

    # All pacakges must have goOffline routine.  Called when the user signs
    # off.  NOT called when the package is unloaded.
    proc goOffline {} {
    }

    # All packages must have a unload routine.  This should remove everything 
    # the package set up.  This is called before load is called when reloading.
    proc unload {} {
        tik_unregister_proxy Socks
    }

    # connect --
    #     Connect via socks proxy.
    #
    # Arguments:
    #     host  - The ip of the host we are connecting to through socks
    #     port  - The port we are connecting to through socks
    #     sname - Our user name, since some proxies might need it.

    proc connect { host port sname } {
        if { ! [info exists ::SSLHOST] || ! [info exists ::SSLPORT]} {
            error "SSL ERROR: Please set SSLHOST and SSLPORT\n"
        }

        set fd [socket $::SSLHOST $::SSLPORT]
        fconfigure $fd -translation binary
        puts -nonewline $fd "CONNECT $host:$port HTTP/1.0\r\n\r\n"
        flush $fd

        set firstline [string trim [gets $fd]]
        puts "SSL: $firstline"
        while {1} {
            set line [string trim [gets $fd]]
            if {$line == ""} {
                break;
            }
            puts "SSL: $line"
        }

        if {[string first "200" $firstline] == -1} {
            error "Some kind of SSL error, check the output."
            return 0
        }
        return $fd
    }

    proc config {} {
        set w .proxyconfig
        destroy $w

        toplevel $w -class Tik
        wm title $w "Proxy Config: SSL/HTTP Connection"
        wm iconname $w "Proxy Config"
        if {$::TIK(options,windowgroup)} {wm group $w .login}
        label $w.label -text "Change your tikrc to make permanent.\n\
             SSL Proxies usually require the TOC Port\n\
             to be set to 443 or 563.\n"

        frame $w.tochostF
        label $w.tochostF.l -text "TOC Host: " -width 15
        entry $w.tochostF.e -textvariable ::TOC($::SELECTEDTOC,host) \
            -exportselection 0
        pack $w.tochostF.l $w.tochostF.e -side left

        frame $w.tocportF
        label $w.tocportF.l -text "TOC Port: " -width 15
        entry $w.tocportF.e -textvariable ::TOC($::SELECTEDTOC,port) \
            -exportselection 0
        pack $w.tocportF.l $w.tocportF.e -side left

        frame $w.sslhostF
        label $w.sslhostF.l -text "SSL Host: " -width 15
        entry $w.sslhostF.e -textvariable ::SSLHOST \
            -exportselection 0
        pack $w.sslhostF.l $w.sslhostF.e -side left

        frame $w.sslportF
        label $w.sslportF.l -text "SSL Port: " -width 15
        entry $w.sslportF.e -textvariable ::SSLPORT \
            -exportselection 0
        pack $w.sslportF.l $w.sslportF.e -side left

        button $w.ok -text "Ok" -command "destroy $w"
        pack $w.label $w.tochostF $w.tocportF \
             $w.sslhostF $w.sslportF $w.ok -side top
    }
}
