# Ticker Package
#
# All packages must be inside a namespace with the
# same name as the file name.
#
# $Revision: 1.15 $

# Options the user might want to set.  A user should use
# set ::TIK(options,...), not the tik_default_set
tik_default_set options,Ticker,on  0
tik_default_set options,Ticker,aim 1
tik_default_set options,Ticker,geometry "+0+0"

namespace eval ticker {

    variable info

    # Must export at least: load, unload, goOnline, goOffline
    namespace export load unload goOnline goOffline setGeometry

    # All packages must have a load routine.  This should do most
    # of the setup for the package.  Called only once.
    proc load {} {
        set ticker::info(font) $::SAGFONT
        set ticker::info(lineheight) [font metrics $ticker::info(font) -linespace]
        set ticker::info(lastbitem) ""
        set ticker::info(bitems) ""

        toc_register_func * UPDATE_BUDDY      ticker::UPDATE_BUDDY
        menu .tickerMenu -tearoff 0
        .toolsMenu add cascade -label "Buddy Ticker" -menu .tickerMenu
        .tickerMenu add checkbutton -label "Enable Buddy Ticker" -onvalue 1 \
                                -offvalue 0 -variable ::TIK(options,Ticker,on)\
                                -command ticker::resetTicker
        .tickerMenu add checkbutton -label "Show AIM Buddies" -onvalue 1 \
                                -offvalue 0 -variable ::TIK(options,Ticker,aim)\
                                -command ticker::resetTicker
    }

    # All pacakges must have goOnline routine.  Called when the user signs
    # on, or if the user is already online when packages loaded.
    proc goOnline {} {
        destroy {.ticker}
        catch {after cancel $ticker::info(timer)}

        if {!$::TIK(options,Ticker,on)} {
            return
        }

        toplevel .ticker -class Tik 
        wm geometry .ticker $::TIK(options,Ticker,geometry)
        wm title .ticker "Buddy Ticker"
        wm iconname .ticker "Buddy Ticker"
        if {$::TIK(options,windowgroup)} {wm group .ticker .login}
        canvas .ticker.c -height [expr $ticker::info(lineheight) * 2]
        pack .ticker.c -expand 1 -fill both 
        after 2000 ticker::createTicker 1
    }

    # All pacakges must have goOffline routine.  Called when the user signs
    # off.  NOT called when the package is unloaded.
    proc goOffline {} {
        catch {after cancel $ticker::info(timer)}
        destroy {.ticker}

        foreach i [array names ticker::info *,online] {
            unset ticker::info($i)
        }
    }

    # All packages must have a unload routine.  This should remove everything 
    # the package set up.  This is called before load is called when reloading.
    proc unload {} {
        catch {destroy .ticker}
        toc_unregister_func * UPDATE_BUDDY      ticker::UPDATE_BUDDY
        .toolsMenu delete "Buddy Ticker"
        destroy .tickerMenu
    }

    proc resetTicker {} {
        set ex [winfo exists .ticker]
        if {$::TIK(options,Ticker,on) && !$ex} {
            goOnline
        } elseif {!$::TIK(options,Ticker,on) && $ex} {
            goOffline
        } else {
            createTicker 1
        }
    }

    proc UPDATE_BUDDY {name user online evil signon idle uclass} {
        set nuser [normalize $user]

        if {(![info exists ticker::info($nuser,online)] ||
                 ($ticker::info($nuser,online) == "F")) && ($online == "T")} {

            set str "$user signed on"
        } elseif {(![info exists ticker::info($nuser,online)] ||
                 ($ticker::info($nuser,online) == "T")) && ($online == "F")} {

            set str "$user signed off"
        } else {
            return
        }

        if {!$::TIK(options,Ticker,on) || ![winfo exists .ticker.c]} {
            return
        }

        set width [winfo width .ticker.c]
        set x 0
        if {$ticker::info(lastbitem) != ""} {
            set x [lindex [.ticker.c coords $ticker::info(lastbitem)] 0]
            set x [expr int($x + [font measure $ticker::info(font) $str])]
            incr x 10
        }
        if {$x < $width} {
            set x $width
        }

        set in [.ticker.c create text $x $ticker::info(lineheight) \
            -text $str\
            -font $ticker::info(font) \
            -anchor ne \
            -fill red -tags BOTTOM]
        set ticker::info($nuser,online) $online
        set ticker::info(lastbitem) $in
        lappend ticker::info(bitems) $in
    }

    proc createTicker {all} {
        if {![winfo exists .ticker.c]} {
            return
        }

        if {$all} {
            set ticker::info(cnt) 0
            .ticker.c delete TOP
        } else {
            set del [expr $ticker::info(cnt) - 1]
            .ticker.c delete TOP$del
            incr ticker::info(cnt)
        }
        set tags [list TOP TOP$ticker::info(cnt)]

        set x [winfo width .ticker.c]
        incr x 20
        set ticker::info(bottomlen) [winfo width .ticker.c]
        set in ""
        foreach g $::BUDDYLIST {
            if {($::GROUPS($g,type) == "AIM") && !$::TIK(options,Ticker,aim)} {
                continue
            }

            incr x [font measure $ticker::info(font) "$g:"]
            set in [ .ticker.c create text $x 0 \
                -text "$g:" \
                -font $ticker::info(font) \
                -anchor ne \
                -fill red \
                -tags $tags]
            
            set ticker::info($in,type) GROUP

            incr x 10
            foreach b $::GROUPS($g,people) {
                if {$::BUDDIES($b,online) == "F"} {
                    continue;
                }
                set bud $::BUDDIES($b,name)
                set other $::BUDDIES($b,otherString)

                incr x 16
                set iconIndex ""
                catch {set iconIndex [.ticker.c create image $x 0 \
                    -image $::BUDDIES($b,icon) \
                    -anchor ne\
                    -tags $tags]}


                incr x [font measure $ticker::info(font) $bud]
                incr x 5
                set budIndex [.ticker.c create text $x 0 \
                    -text $bud \
                    -font $ticker::info(font) \
                    -anchor ne \
                    -fill black\
                    -tags $tags]

                incr x [font measure $ticker::info(font) $other]
                incr x 5
                set in [.ticker.c create text $x 0 \
                    -text $other \
                    -font $ticker::info(font) \
                    -anchor ne \
                    -fill blue\
                    -tags $tags]
                incr x 10

                set ticker::info($in,type) OTHER
                set ticker::info($in,b) $b
                .ticker.c bind $budIndex <Double-Button-1> "tik_double_click $b $b"
                .ticker.c bind $in <Double-Button-1> "tik_double_click $b $b"
                .ticker.c bind $budIndex <ButtonPress-3> "tik_buddy_popup $b %X %Y"
                .ticker.c bind $budIndex <ButtonRelease-3> {tik_buddy_release}
                .ticker.c bind $in <ButtonPress-3> "tik_buddy_popup $b %X %Y"
                .ticker.c bind $in <ButtonRelease-3> {tik_buddy_release}
            }
        }
        catch {after cancel $ticker::info(timer)}
        set ticker::info(timer) [after 100 ticker::updateTicker]
        set ticker::info(lastitem) $in
    }

    proc updateTicker {} {
        if {$ticker::info(lastitem) != ""} {
            set width [winfo width .ticker.c]
            set x [lindex [.ticker.c coords $ticker::info(lastitem)] 0]
            if {$x <= $width } {
                createTicker 0
                return
            }
            set ticker::info(timer) [after 100 ticker::updateTicker]
            .ticker.c move TOP -5 0
        }

        set bitems ""
        foreach bitem $ticker::info(bitems) {
            set x [lindex [.ticker.c coords $bitem] 0]
            if {$x < 0} {
                .ticker.c delete $bitem
                if {$bitem == $ticker::info(lastbitem)} {
                    set ticker::info(lastbitem) ""
                }
            } else {
                lappend bitems $bitem
            }
        }
        set ticker::info(bitems) $bitems
        .ticker.c move BOTTOM -3 0
    }
}
