#! /bin/sh
# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

# Copyright (c) 1998 America Online, Inc. All Rights Reserved.
#
# AOL grants you ("Licensee") a non-exclusive, royalty free, license to use,
# modify and redistribute this software in source and binary code form,
# provided that i) this copyright notice and license appear on all copies of
# the software; and ii) Licensee does not utilize the software in a manner
# which is disparaging to AOL.
#
# This software is provided "AS IS," without a warranty of any kind. ALL
# EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
# IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
# NON-INFRINGEMENT, ARE HEREBY EXCLUDED. AOL AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL AOL OR ITS
# LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
# INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
# CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
# OR INABILITY TO USE SOFTWARE, EVEN IF AOL HAS BEEN ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGES.
#
# This software is not designed or intended for use in on-line control of
# aircraft, air traffic, aircraft navigation or aircraft communications; or in
# the design, construction, operation or maintenance of any nuclear
# facility. Licensee represents and warrants that it will not use or
# redistribute the Software for such purposes.

# TAC.TCL --
# A Tcl only version of TiK that has a IRC ish interface.  Because
# this is in Tcl and doesn't use curses, the line editing doesn't
# work very well on some platforms.
#
# Bugs: tictoc@aol.net   (please check FAQs on web page first)
# Home Page: http://www.aim.aol.com/tik

# TODO:
# * Support more command, especially with chat
# * Document, Document, Document

set VERSION "Tac 0.57"
set REVISION {TAC:$Revision: 1.22 $}

# Make sure we are in the tik directory and load the toc routines.
cd [file dirname $argv0]
catch {cd [file dirname [file readlink $argv0]]}
source toc.tcl

set TAC(input) 0
set TAC(output) ""

################
proc unHTML {msg} {
# ugh. aim is in HTML. we hates it my precious.
    foreach e [splitHTML $msg] {
        if {![regexp -- {^<.*>$} $e match tag]} {
            append text $e
        }
    }
    return $text
}

#******************************************************
#*******************INCOMING PROTOCOL******************
#******************************************************

################
proc SIGN_ON {name version} {
    # The following is true after migration
    if {[llength $::BUDDYLIST] > 0} {  
        tac_send_init
    }

    tac_output "...Welcome to AOL Instant Messanger ($::VERSION)..."
}

################
proc CONFIG {name data} {
    set lines [split $data "\n"]
    foreach i $lines {
        switch -exact -- [string index $i 0] {
        "b" {
            set bud [normalize [string range $i 2 end]]
            set ::BUDDIES($bud,online) F
            set ::BUDDIES($bud,name) $bud
            set ::BUDDIES($bud,idle) 0
            lappend ::GROUPS($group,people) $bud
        } 
        "d" {
            set deny [string range $i 2 end]
            lappend ::DENYLIST $deny
        }
        "g" {
            set group [string range $i 2 end]
            lappend ::BUDDYLIST $group
            lappend ::GROUPS($group,collapsed) F
            set ::GROUPS($group,people) [list]
        }
        "m" {
            set ::PDMODE [string range $i 2 end]
        }
        "p" {
            set permit [string range $i 2 end]
            lappend ::PERMITLIST $permit
        }
        }
    }

    tac_send_init
}

################
proc NICK {name nick} {
    set ::SCREENNAME $nick
}


################
proc IM_IN {name source msg auto} {
    set text [unHTML $msg]
    tac_output "<*$source*> $text"
}

################
proc UPDATE_BUDDY {name user online evil signon idle uclass} {
    set bud [normalize $user]
    set ::BUDDIES($bud,name) $user
    set o $::BUDDIES($bud,online)
    set ::BUDDIES($bud,online) $online
    set ::BUDDIES($bud,evil) $evil
    set ::BUDDIES($bud,signon) $signon
    set i $::BUDDIES($bud,idle)
    set ::BUDDIES($bud,idle) $idle
    if { ($o != $online) } {
        if {$online == "T"} {
            tac_output "** $user has signed on."
        } else {
            tac_output "** $user has signed off."
        }
    }
}

################
proc ERROR {name code data} {
    set args [split $data ":"]
    switch -exact -- $code {
    "901" {
        tac_output "[lindex $args 0] not currently available."
    } 
    "902" {
        tac_output \
            "Warning of [lindex $args 0] is not allowed."
    } 
    "903" {
        tac_output "A message has been dropped, you\
                                         are exceeding the server speed limit."
    } 
    "950" {
        tac_output \
            "Chat in [lindex $args 0] is unavailable."
    }
    "960" {
        tac_output \
            "You are sending message too fast to [lindex $args 0]."
    }
    "961" {
        tac_output \
            "You missed an im from [lindex $args 0] because it was too big."
    }
    "962" {
        tac_output "You missed an im from\
                               [lindex $args 0] because it was sent too fast."
    }
    "980" {
        tac_output "Incorrect nickname or password."
    }
    "981" {
        tac_output "The service is temporarly unavailable."
    }
    "982" {
        tac_output \
            "Your warning level is currently too high to sign on."
    }
    "983" {
        tac_output "You have been connecting and\
            disconnecting too frequently.  Wait 10 minutes and try again.\
            If you continue to try, you will need to wait even longer."
    }
    "989" {
        tac_output \
            "An unknown signon error has occured. ([lindex $args 0])"
    }
    default {
        tac_output "Unknown error $code:$data"
    }
    } ;# SWITCH
}

################
proc EVILED {name level user} {
    if {[string length $user] == 0 } {
        tac_output \
            "You were warned anonymously! Your new warning level is $level%."
    } else {
        tac_output \
            "You were warned by $user! Your new warning level is $level%."
    }
}

proc GOTO_URL {name window url} {
    set toc $::SELECTEDTOC

    tac_output "Goto the url $url"
}

################
proc PAUSE {name data} {
    tac_output "PAUSING"
}

################
proc CHAT_INVITE {name loc id sender msg} {
    tac_output "$sender invited you to chat room '$loc' with id '$id'"
    tac_output "Invitation message is '$msg'"
    tac_output "To accept, type /ca $id"
}

################
proc CHAT_JOIN {name id loc} {
    catch {
        set people $::INVITES($loc,people)
        set msg $::INVITES($loc,msg)

        set p ""
        foreach i [split $people "\n" ] {
            set n [normalize $i]
            if {$n != "" } {
                append p $n
                append p " "
            }
        }

        if {$p != ""} {
            toc_chat_invite $name $id $msg $p
        }

        unset ::INVITES($loc,people)
        unset ::INVITES($loc,msg)
    }

    # at this point, the other guy sees us join
    tac_output "You have joined chat room '$loc' with id '$id'."
}

################
proc CHAT_LEFT {name id} {
    tac_output "You have left chat room id '$id'"
}

################
proc CHAT_IN {name id source whisper msg} {
    tac_receive_chat $id $source $whisper $msg
}

################
proc CONNECTION_CLOSED {name data} {
    tac_output "Connection Closed"
    exit 0
}

#******************************************************
#********************** TAC METHODS *******************
#******************************************************


################
proc tac_receive_chat {id remote whisper msg} {

    set whisperstr ""
    if { $whisper == "T" } {
        set whisperstr " (whispers) "
    }

    set text [unHTML $msg] 
    tac_output "<$id:$remote>$whisperstr $text"
}

################
proc tac_output {str} {
    if {$::TAC(input)} {
        append ::TAC(output) $str
        append ::TAC(output) "\r\n"
    } else {
        puts -nonewline "$str\r\n"
    }
}

################
proc tac_set_config {} {
    set str ""
    append str "m $::PDMODE\n"
    foreach p $::PERMITLIST {
        append str "p $p\n"
    }
    foreach d $::DENYLIST {
        append str "d $d\n"
    }
    foreach g $::BUDDYLIST {
        append str "g $g\n"
        foreach b $::GROUPS($g,people) {
            append str "b $b\n"
        }
    }

    toc_set_config $::NSCREENNAME $str
}

################
proc tac_add_buddy {group name} {
    if {[catch {set t $::BUDDIES($name,online)}] != 0} {
        set ::BUDDIES($name,online) F
        set ::BUDDIES($name,name) $name
        set ::BUDDIES($name,idle) 0
        toc_add_buddy $::NSCREENNAME $name
    }

    if {[catch {set t $::GROUPS($group,people)}] != 0} {
        set ::GROUPS($group,people) [list]
        set ::GROUPS($group,collapsed) F
        lappend ::BUDDYLIST $group
        lappend ::GROUPS($group,people) $name
    } else {
        lappend ::GROUPS($group,people) $name
    }
}

################
proc tac_send_init {} {
    foreach g $::BUDDYLIST {
        foreach b $::GROUPS($g,people) {
            lappend buds $b
        }
    }

    if {[info exists buds] == 0} {
        tac_add_buddy Buddies [normalize $::SCREENNAME]
    } else {
        toc_add_buddy $::SCREENNAME $buds
    }

    if {$::PDMODE == "3"} {
        toc_add_permit $::SCREENNAME $::PERMITLIST
    } elseif {$::PDMODE == "4"} {
        toc_add_deny $::SCREENNAME $::DENYLIST
    }

    toc_init_done $::SCREENNAME
}

################
proc tac_connect {} {
    set ::BUDDYLIST [list]
    set ::PERMITLIST [list]
    set ::DENYLIST [list]
    set ::PDMODE 1

    if {![info exists ::SCREENNAME]} {
        puts -nonewline "login: "
        flush stdout
        set ::SCREENNAME [string trimright [gets stdin]]
    }
    set ::NSCREENNAME [normalize $::SCREENNAME]
    if {![info exists ::PASSWORD]} {
        puts -nonewline "Password: "
        flush stdout
        exec stty -echo
        set ::PASSWORD [string trimright [gets stdin]]
        exec stty echo
        puts ""
    }
    flush stdout
    toc_open $::NSCREENNAME toc.oscar.aol.com 5190 login.oscar.aol.com \
             5190 $::NSCREENNAME $::PASSWORD english $::REVISION
    unset ::PASSWORD
}

################
proc tac_set_terminal {mode} {
    if {$mode == "cooked"} {
        fconfigure stdin -blocking true
        fconfigure stdin -buffering line
        fileevent stdin readable {}
        exec stty cooked
    } elseif {$mode == "raw"} {
        fconfigure stdin -blocking false
        fconfigure stdin -buffering none
        exec stty raw
        fileevent stdin readable tac_handle_input
    }
}
################
proc tac_register_cmd {cmdexp function {help {}}} {
    lappend ::TAC(commands) $cmdexp
    set ::TAC(commands,$cmdexp,func) $function
    set ::TAC(commands,$cmdexp,help) $help
}
################
proc tac_unregister_cmd {cmdexp} {
    set pos [lsearch -exact $::TAC(commands) $cmdexp]
    set ::TAC(commands) [lreplace $::TAC(commands) $pos $pos]
    unset ::TAC(commands,$cmdexp,func)
    unset ::TAC(commands,$cmdexp,help)
}
################
proc tac_get_help {cmdexp} {
    return $::TAC(commands,$cmdexp,help)
}

################
proc tac_handle_input {} {
    set ::TAC(input) 1
    tac_set_terminal cooked
    set line [string trimright [gets stdin]]

    set found 0
    foreach cmdexp $::TAC(commands) {
        if {[regexp -nocase -- $cmdexp $line]} {
            set found 1
            $::TAC(commands,$cmdexp,func) $cmdexp $line 
        }
    }

    if {!$found} {
        tac_output ""
        tac_output "Unknown command $line"
        tac_output "Valid commands:"
        foreach cmdexp $::TAC(commands) {
            if {$::TAC(commands,$cmdexp,help) != ""} {
                tac_output $::TAC(commands,$cmdexp,help)
            }
        }
    }

    if {$::TAC(output) != ""} {
        puts -nonewline "$::TAC(output)"
        set ::TAC(output) ""
    }
    set ::TAC(input) 0

    tac_set_terminal raw
}
#******************************************************
#********************USER COMMANDS*********************
#******************************************************
proc tac_cmd_who {exp line} {
    tac_output "  User\t\t\tIdle\tEvil"
    tac_output "------------------------------------"
    foreach g $::BUDDYLIST {
        tac_output $g
        foreach b $::GROUPS($g,people) {
            set o $::BUDDIES($b,online)
            if { ($o == "T") } { 
                set out "  $::BUDDIES($b,name)\t"
                if {[string length $::BUDDIES($b,name)] < 6} {
                    append out "\t\t"
                } elseif {[string length $::BUDDIES($b,name)] < 14} {
                    append out "\t"
                }
                tac_output "$out$::BUDDIES($b,idle)\t$::BUDDIES($b,evil)%"
            }
        }
    }
}

################
proc tac_cmd_add {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) (.*)$} $line match cmd name group]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }
    tac_add_buddy $group $name
    tac_set_config
}

################
proc tac_cmd_im {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) (.*)$} $line match cmd name msg]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_send_im $::NSCREENNAME $name $msg
}

################
proc tac_cmd_evil {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) *$} $line match cmd name]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_evil $::NSCREENNAME $name
}

################
proc tac_cmd_quit {exp line} {
    set ::DONE 1
}

################
proc tac_cmd_chat_accept {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([0-9]+) *$} $line match cmd id]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }
    tac_output "DEBUG: sending accept message for chat $id"
    toc_chat_accept $::NSCREENNAME $id
}

################
proc tac_cmd_chat_msg {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([0-9]+) (.*)$} $line match cmd id msg]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_chat_send $::NSCREENNAME $id $msg
}

################
proc tac_cmd_chat_evil {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([0-9]+) ([a-zA-Z0-9]+)$} $line match cmd id name]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_chat_evil $::NSCREENNAME $id $name
}

################
proc tac_cmd_chat_who {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([0-9]+) *$} $line match cmd id]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    tac_output "DEBUG: Not implemented yet."
}

################
proc tac_cmd_sdefault {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) *$} $line match cmd name]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set ::TAC(defbud) $name
    tac_output "Default buddy is now $::TAC(defbud)."
}

################
proc tac_cmd_default {exp line} {
    if {![regexp -- {^(\.) (.*)$} $line match cmd msg]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    if {"$::TAC(defbud)"==""} { tac_output "No default buddy"; return }
    toc_send_im $::NSCREENNAME $::TAC(defbud) $msg
}

#******************************************************
#********************** MAIN **************************
#******************************************************

toc_register_func * SIGN_ON           SIGN_ON
toc_register_func * CONFIG            CONFIG
toc_register_func * NICK              NICK
toc_register_func * IM_IN             IM_IN
toc_register_func * UPDATE_BUDDY      UPDATE_BUDDY
toc_register_func * ERROR             ERROR
toc_register_func * EVILED            EVILED
toc_register_func * GOTO_URL          GOTO_URL
toc_register_func * PAUSE             PAUSE
toc_register_func * CHAT_INVITE       CHAT_INVITE
toc_register_func * CHAT_JOIN         CHAT_JOIN
toc_register_func * CHAT_LEFT         CHAT_LEFT
#toc_register_func * CHAT_UPDATE_BUDDY CHAT_UPDATE_BUDDY
toc_register_func * CONNECTION_CLOSED CONNECTION_CLOSED
toc_register_func * CHAT_IN           CHAT_IN

tac_register_cmd "^/m "  tac_cmd_im             "  /m user message send an im"
tac_register_cmd "^/e "  tac_cmd_evil           "  /e user         warn a user"
tac_register_cmd "^/w$"  tac_cmd_who            "  /w              show currently logged in buddies"
tac_register_cmd "^/a "  tac_cmd_add            "  /a user group   add buddy to group"
tac_register_cmd "^/ca " tac_cmd_chat_accept    "  /ca id          accept chat id"
tac_register_cmd "^/cm " tac_cmd_chat_msg       "  /cm id msg      send msg to chat id"
tac_register_cmd "^/ce " tac_cmd_chat_evil      "  /ce id user     warn a user in a chat room"
tac_register_cmd "^/cw " tac_cmd_chat_who       "  /cw id          show users in a a chat room"
tac_register_cmd "^/q$"  tac_cmd_quit           "  /q              log out"
tac_register_cmd "^/s "  tac_cmd_sdefault       "  /s user         set default buddy"
tac_register_cmd "^\. "  tac_cmd_default        "  . message       send default message"

# Globals and Options
set TAC(rcfile) ~/.tacrc
set TAC(defbud) ""

# Set Terminal to normal state
fconfigure stdin -translation binary
tac_set_terminal cooked

# Parse Options
proc argCheck {arg i l} {
    if {$i >= $l} {
        puts "$::argv0: Missing argument for $arg, try -H for usage."
        exit 1;
    }
}
set largv [llength $argv]
for {set i 0} {$i < $largv} {incr i} {
    set arg [lindex $argv $i]
    switch -glob -- $arg {
    "-H" -
    "-h" -
    "-help" {
        puts "$argv0 Usage:"
        puts " -H             This message"
        puts " -sflap <level> SFLAP debug level"
        puts " -rcfile <fn>   Use <fn> instead of $::TAC(rcfile)"
        puts " -user <user>   Use <user>, overrides rc files"
        puts " -pass <pass>   User <pass>, overrides rc files"
        puts " -roast <pass>  Roast a password for use in ~/.tacrc or -pass"
        exit 0
    }
    "-sflap" {
        incr i
        argCheck $arg $i $largv
        set sflap::debug_level [lindex $argv $i]
    }
    "-rcfile" {
        incr i
        argCheck $arg $i $largv
        set ::TAC(rcfile) [lindex $argv $i]
    }
    "-u" - 
    "-user" {
        incr i
        argCheck $arg $i $largv
        set SCREENNAME [set ::TAC(clUser) [lindex $argv $i]]
    }
    "-p" - 
    "-pass" {
        incr i
        argCheck $arg $i $largv
        set PASSWORD [set ::TAC(clPass) [lindex $argv $i]]
    }
    "-roast" {
        incr i
        argCheck $arg $i $largv
        puts -nonewline "\nPlace \"set PASSWORD "
        puts "0x[roast_password [lindex $argv $i]]\" in your $::TAC(rcfile) file.\n"
        puts -nonewline "WARNING: While not in clear text, people can still "
        puts "decode your password."
        puts "You should do a \"chmod og-rwx $::TAC(rcfile)\" for some protection."
        exit 0
    }
    default {
        puts "$::argv0: Unknown argument '$arg', try -H for usage."
    }
    } ;# SWITCH
}

# Load the user config
if {[file exists $::TAC(rcfile)] == 1} {
    source $::TAC(rcfile)
}

catch {set SCREENNAME $TIK(clUser)}
catch {set PASSWORD $TIK(clPass)}

tac_connect

tac_set_terminal raw

vwait DONE

tac_set_terminal cooked
exit 0
