# Stock Monitoring Package
#
# This is a proof of concept module.  We just fetch the data
# off a web page.
#
# All packages must be inside a namespace with the
# same name as the file name.
#
# $Revision: 1.10 $

package require http 2.0

# Options the user might want to set.  A user should use
# set ::TIK(options,...), not the tik_default_set
tik_default_set options,Stock,update 300000

namespace eval stock {

    variable info

    # Must export at least: load, unload, goOnline, goOffline
    namespace export load unload goOnline goOffline add

    # All packages must have a load routine.  This should do most
    # of the setup for the package.  Called only once.
    proc load {} {
        # We use TIK variable to store information, so it isn't
        # lost during reloads.
        if {![info exists ::TIK(Stocks,list)]} {
            set ::TIK(Stocks,list) ""
        }

        if {$::TIK(options,Stock,update) < 60000} {
            set ::TIK(options,Stock,update) 60000
        }

        set stock::info(timer)       ""

        menu .stockMenu -tearoff 0
        .toolsMenu add cascade -label "Stock Monitor" -menu .stockMenu
        .stockMenu add command -label "Update Now" -command stock::doUpdate
        .stockMenu add command -label "New Stock" -command stock::newStock
        menu .stockIMenu -tearoff 0
        .stockMenu add cascade -label "Update Interval" -menu .stockIMenu
        .stockIMenu add radiobutton -label "5 Minutes" -value 300000 \
            -variable ::TIK(options,Stock,update) -command stock::doUpdate
        .stockIMenu add radiobutton -label "10 Minutes" -value 600000 \
            -variable ::TIK(options,Stock,update) -command stock::doUpdate
        .stockIMenu add radiobutton -label "15 Minutes" -value 900000 \
            -variable ::TIK(options,Stock,update) -command stock::doUpdate
        .stockIMenu add radiobutton -label "30 Minutes" -value 1800000 \
            -variable ::TIK(options,Stock,update) -command stock::doUpdate
    }

    proc addStock {stock name symbol} {
        lappend ::GROUPS(Stocks,people) $stock
        set ::BUDDIES($stock,type) Stock
        set ::BUDDIES($stock,online) T
        set ::BUDDIES($stock,name) $name
        set ::BUDDIES($stock,symbol) $symbol
        set ::BUDDIES($stock,otherString) "*NOT UPDATED*"
        set ::BUDDIES($stock,indexs) ""
        set ::BUDDIES($stock,popupText) ""
        set ::BUDDIES($stock,doubleClick) "stock::doubleClick"
        set ::BUDDIES($stock,icon) ""
        incr ::GROUPS(Stocks,online)
        incr ::GROUPS(Stocks,total)
    }

    proc doubleClick {name stock} {
        set symbol $::BUDDIES([normalize $stock],symbol)
        tik_show_url StockWin "http://quicken.aol.com/investments/quotes/?symbol=$symbol"
    }

    # All pacakges must have goOnline routine.  Called when the user signs
    # on, or if the user is already online when packages loaded.
    proc goOnline {} {
        lappend ::BUDDYLIST Stocks
        set ::GROUPS(Stocks,people) ""
        set ::GROUPS(Stocks,type) Stocks
        set ::GROUPS(Stocks,online) 0
        set ::GROUPS(Stocks,total) 0
        set ::GROUPS(Stocks,collapsed) F

        foreach stock $::TIK(Stocks,list) {
            set name $::TIK(Stocks,$stock,name)
            set symbol $::TIK(Stocks,$stock,symbol)

            addStock $stock $name $symbol
        }

        tik_draw_list T
        doUpdate
    }

    # All pacakges must have goOffline routine.  Called when the user signs
    # off.  NOT called when the package is unloaded.
    proc goOffline {} {
    }

    # All packages must have a unload routine.  This should remove everything 
    # the package set up.  This is called before load is called when reloading.
    proc unload {} {
        catch {after cancel $stock::info(timer)}
        .toolsMenu delete "Stock Monitor"
        destroy .stockIMenu
        destroy .stockMenu
        catch {destroy .newStock}
        set i [lsearch -exact $::BUDDYLIST Stocks]
        if {$i != -1} {
            set ::BUDDYLIST [lreplace $::BUDDYLIST $i $i]
        }
        tik_draw_list T
    }

    proc add {name symbol} {
        set stock [normalize $name]

        if {[lsearch $::TIK(Stocks,list) $stock] != -1} {
            return
        }

        lappend ::TIK(Stocks,list) $stock
        set ::TIK(Stocks,$stock,symbol) [string tolower $symbol]
        set ::TIK(Stocks,$stock,name) $name
    }

    proc doUpdate {} {
        catch {after cancel $stock::info(timer)}
        set stock::info(timer) [after $::TIK(options,Stock,update) stock::doUpdate]

        if {($::USEPROXY != "None") && !$::TIK(options,HTTPProxy,enable)} {
            return
        }

        aolDoUpdate
    }


# CODE FOR AOL STOCKS
    proc aolDoUpdate {} {
        if {[llength $::TIK(Stocks,list)] == 0} {
            return
        }

        set url "http://quicken.aol.com/investments/quotes/?symbol="

        foreach s $::TIK(Stocks,list) {
            append url "$::TIK(Stocks,$s,symbol)+"
        }

        if {[llength $::TIK(Stocks,list)] == 1} {
            append url "aol"
        }

        catch {::http::geturl $url -command stock::aolDataAvailable}
    }


    # Callback when there is data available from the url we have
    # requested.  This routine parses the results from aol
    # and updates the display.
    proc aolDataAvailable {token} {
        upvar #0 $token state

        regsub -all -- {<SU[PB]>} $state(body) "" body
        regsub -all -- {</SU[PB]>} $body "" body
        regsub -all -- "(\[-+])\t" $body {\1} body
        regsub -all -- "\t" $body " " body
        regsub -all -- "\n" $body " " body

        set items [splitHTML $body]
        set len [llength $items]
        for {set i 0} {$i < $len} {incr i} {
            set item [lindex $items $i]
            if {$item == "More Info"} {
                break;
            }
        }

        for {} {$i < $len} {incr i} {
            set item [lindex $items $i]
            if {[string match "<TR*" $item]} {
                break;
            }
        }

        set clist ""
        set cstr ""
        for {} {$i < $len} {incr i} {
            set item [string trim [lindex $items $i]]
            if {$item == ""}  {
            } elseif {$item == "</TR>"} {
                catch {
                set symbol [string tolower [lindex $clist 0]]
                set value [lindex $clist 1]
                set change [lindex $clist 2]

                set stock ""
                foreach s $::TIK(Stocks,list) {
                    if {$::TIK(Stocks,$s,symbol) == $symbol} {
                        set stock $s
                        break
                    }
                }

                if {$stock == ""} continue

                set ::BUDDIES($stock,popupText) [list \
                    Symbol: [lindex $clist 0] \
                    Last: [lindex $clist 1] \
                    Change: [lindex $clist 2] \
                    Volume: [lindex $clist 3] \
                    Bid: [lindex $clist 4] \
                    Ask: [lindex $clist 5] \
                    Time: [lindex $clist 6] \
                    Market: [lindex $clist 7]]

                set clist ""

                set ::BUDDIES($stock,otherString) "$value \[$change\]"
                sag::change_otherstring .buddy.list $::BUDDIES($stock,indexs) \
                    "$value \[$change\]"

                if {[string index $change 0] == "-"} {
                    set ::BUDDIES($stock,icon) downarrow
                    sag::change_icon .buddy.list $::BUDDIES($stock,indexs) downarrow
                } elseif {[string index $change 0] == "+"} {
                    set ::BUDDIES($stock,icon) uparrow
                    sag::change_icon .buddy.list $::BUDDIES($stock,indexs) uparrow
                } else {
                    set ::BUDDIES($stock,icon) ""
                    sag::change_icon .buddy.list $::BUDDIES($stock,indexs) ""
                }
                } 
            } elseif {$item == "</TD>"} {
                lappend clist $cstr
                set cstr ""
            } elseif {$item == "</TABLE>"} {
                break;
            } elseif {[string index $item 0] != "<"} {
                if {$cstr != ""} {
                    append cstr " "
                }
                append cstr $item
            }
        }
    }

# CODE FOR ADDING A NEW STOCK

    proc newStock_ok {} {
        set stock [normalize $stock::info(NEW,name)]

        add $stock::info(NEW,name) $stock::info(NEW,symbol)
        addStock $stock $stock::info(NEW,name) $stock::info(NEW,symbol)

        tik_draw_list T
        doUpdate
    }

    proc newStock {} {
        set w .newStock

        if {[winfo exists $w]} {
            raise $w
            return
        }

        toplevel $w -class Tik
        wm title $w "New Stock"
        wm iconname $w "New Stock"
        if {$::TIK(options,windowgroup)} {wm group $w .login}
        set stock::info(NEW,name) ""
        set stock::info(NEW,symbol) ""

        label $w.info -text "This will NOT change your $::TIK(rcfile)\n Use: stock::add <name> <symbol>"

        frame $w.nameF
        label $w.nameL -text "Enter Stock Name: " -width 20
        entry $w.name  -textvariable stock::info(NEW,name)
        pack $w.nameL $w.name -in $w.nameF -side left
        bind $w.name <Return> "focus $w.symbol"

        frame $w.symbolF
        label $w.symbolL -text "Enter Stock Symbol: " -width 20
        entry $w.symbol  -textvariable stock::info(NEW,symbol)
        pack $w.symbolL $w.symbol -in $w.symbolF -side left
        bind $w.symbol <Return> "destroy $w; stock::newStock_ok"


        frame $w.buttons
        button $w.ok -text "Ok" -command "destroy $w; stock::newStock_ok"
        button $w.cancel -text "Cancel" -command [list destroy $w]
        pack $w.ok $w.cancel -in $w.buttons -side left -padx 2m

        pack $w.info $w.nameF $w.symbolF -side top
        pack $w.buttons -side bottom
        focus $w.name

    }
}
